/* TimedQueue.c generated by valac 0.56.18, the Vala compiler
 * generated from TimedQueue.vala, do not modify */

/*
* Copyright (c) 2010-2013 Yorba Foundation
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/
/* TimedQueue is a specialized collection class.  It holds items in order, but rather than being*/
/* manually dequeued, they are dequeued automatically after a specified amount of time has elapsed*/
/* for that item.  As of today, it's possible the item will be dequeued a bit later than asked*/
/* for, but it will never be early.  Future implementations might tighten up the lateness.*/
/**/
/* The original design was to use a signal to notify when an item has been dequeued, but Vala has*/
/* a bug with passing an unnamed type as a signal parameter:*/
/* https://bugzilla.gnome.org/show_bug.cgi?id=628639*/
/**/
/* The rate the items come off the queue can be spaced out.  Note that this can cause items to back*/
/* up.  As of today, TimedQueue makes no effort to combat this.*/

#include "io.elementary.photos.h"
#include <glib.h>
#include <gee.h>
#include <glib-object.h>
#include <gobject/gvaluecollector.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif

#define TIMED_QUEUE_TYPE_ELEMENT (timed_queue_element_get_type ())
#define TIMED_QUEUE_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElement))
#define TIMED_QUEUE_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementClass))
#define TIMED_QUEUE_IS_ELEMENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TIMED_QUEUE_TYPE_ELEMENT))
#define TIMED_QUEUE_IS_ELEMENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TIMED_QUEUE_TYPE_ELEMENT))
#define TIMED_QUEUE_ELEMENT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElementClass))

typedef struct _TimedQueueElement TimedQueueElement;
typedef struct _TimedQueueElementClass TimedQueueElementClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _TimedQueueElementPrivate TimedQueueElementPrivate;
#define _timed_queue_element_unref0(var) ((var == NULL) ? NULL : (var = (timed_queue_element_unref (var), NULL)))
typedef struct _TimedQueueParamSpecElement TimedQueueParamSpecElement;
typedef struct _ParamSpecTimedQueue ParamSpecTimedQueue;
enum  {
	TIMED_QUEUE_PAUSED_SIGNAL,
	TIMED_QUEUE_NUM_SIGNALS
};
static guint timed_queue_signals[TIMED_QUEUE_NUM_SIGNALS] = {0};
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);
#define _vala_return_if_fail(expr, msg) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return; }
#define _vala_return_val_if_fail(expr, msg, val) if G_LIKELY (expr) ; else { g_return_if_fail_warning (G_LOG_DOMAIN, G_STRFUNC, msg); return val; }
#define _vala_warn_if_fail(expr, msg) if G_LIKELY (expr) ; else g_warn_message (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

struct _TimedQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	guint hold_msec;
	DequeuedCallback callback;
	gpointer callback_target;
	GeeEqualDataFunc equal_func;
	gpointer equal_func_target;
	GDestroyNotify equal_func_target_destroy_notify;
	gint priority;
	guint timer_id;
	SortedList* queue;
	guint dequeue_spacing_msec;
	gint64 last_dequeue;
	gboolean paused_state;
};

struct _TimedQueueElement {
	GTypeInstance parent_instance;
	volatile int ref_count;
	TimedQueueElementPrivate * priv;
	gpointer item;
	gint64 ready;
};

struct _TimedQueueElementClass {
	GTypeClass parent_class;
	void (*finalize) (TimedQueueElement *self);
};

struct _TimedQueueElementPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _TimedQueueParamSpecElement {
	GParamSpec parent_instance;
};

struct _ParamSpecTimedQueue {
	GParamSpec parent_instance;
};

struct _HashTimedQueuePrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeHashMap* item_count;
};

static gint TimedQueue_private_offset;
static gpointer timed_queue_parent_class = NULL;
static gint TimedQueueElement_private_offset;
static gpointer timed_queue_element_parent_class = NULL;
static gint HashTimedQueue_private_offset;
static gpointer hash_timed_queue_parent_class = NULL;

static gpointer timed_queue_element_ref (gpointer instance);
static void timed_queue_element_unref (gpointer instance);
static GParamSpec* timed_queue_param_spec_element (const gchar* name,
                                            const gchar* nick,
                                            const gchar* blurb,
                                            GType object_type,
                                            GParamFlags flags) G_GNUC_UNUSED ;
static void timed_queue_value_set_element (GValue* value,
                                    gpointer v_object) G_GNUC_UNUSED ;
static void timed_queue_value_take_element (GValue* value,
                                     gpointer v_object) G_GNUC_UNUSED ;
static gpointer timed_queue_value_get_element (const GValue* value) G_GNUC_UNUSED ;
static GType timed_queue_element_get_type (void) G_GNUC_CONST  G_GNUC_UNUSED ;
static gint64 timed_queue_element_comparator (void* a,
                                       void* b);
static gint64 _timed_queue_element_comparator_comparator (void* a,
                                                   void* b,
                                                   gpointer self);
static guint timed_queue_get_heartbeat_timeout (TimedQueue* self);
static gboolean timed_queue_on_heartbeat (TimedQueue* self);
static gboolean _timed_queue_on_heartbeat_gsource_func (gpointer self);
static void timed_queue_real_notify_dequeued (TimedQueue* self,
                                       gconstpointer item);
static void timed_queue_real_clear (TimedQueue* self);
static gboolean timed_queue_real_contains (TimedQueue* self,
                                    gconstpointer item);
static gboolean timed_queue_real_enqueue (TimedQueue* self,
                                   gconstpointer item);
static gint64 timed_queue_calc_ready_time (TimedQueue* self);
static TimedQueueElement* timed_queue_element_new (GType g_type,
                                            GBoxedCopyFunc g_dup_func,
                                            GDestroyNotify g_destroy_func,
                                            gconstpointer item,
                                            gint64 ready);
static TimedQueueElement* timed_queue_element_construct (GType object_type,
                                                  GType g_type,
                                                  GBoxedCopyFunc g_dup_func,
                                                  GDestroyNotify g_destroy_func,
                                                  gconstpointer item,
                                                  gint64 ready);
static gboolean timed_queue_real_enqueue_many (TimedQueue* self,
                                        GeeCollection* items);
static gboolean timed_queue_real_remove_first (TimedQueue* self,
                                        gconstpointer item);
static void timed_queue_real_paused (TimedQueue* self,
                              gboolean is_paused);
static void timed_queue_element_finalize (TimedQueueElement * obj);
static GType timed_queue_element_get_type_once (void);
static void timed_queue_finalize (TimedQueue * obj);
static GType timed_queue_get_type_once (void);
static void hash_timed_queue_real_notify_dequeued (TimedQueue* base,
                                            gconstpointer item);
static void hash_timed_queue_removed (HashTimedQueue* self,
                               gconstpointer item);
static void hash_timed_queue_real_clear (TimedQueue* base);
static gboolean hash_timed_queue_real_contains (TimedQueue* base,
                                         gconstpointer item);
static gboolean hash_timed_queue_real_enqueue (TimedQueue* base,
                                        gconstpointer item);
static gboolean hash_timed_queue_real_enqueue_many (TimedQueue* base,
                                             GeeCollection* items);
static gboolean hash_timed_queue_real_remove_first (TimedQueue* base,
                                             gconstpointer item);
static void hash_timed_queue_finalize (TimedQueue * obj);
static GType hash_timed_queue_get_type_once (void);

static inline gpointer
timed_queue_get_instance_private (TimedQueue* self)
{
	return G_STRUCT_MEMBER_P (self, TimedQueue_private_offset);
}

static gint64
_timed_queue_element_comparator_comparator (void* a,
                                            void* b,
                                            gpointer self)
{
	gint64 result;
	result = timed_queue_element_comparator (a, b);
#line 77 "../src/TimedQueue.vala"
	return result;
#line 214 "TimedQueue.c"
}

static gboolean
_timed_queue_on_heartbeat_gsource_func (gpointer self)
{
	gboolean result;
	result = timed_queue_on_heartbeat ((TimedQueue*) self);
#line 79 "../src/TimedQueue.vala"
	return result;
#line 224 "TimedQueue.c"
}

TimedQueue*
timed_queue_construct (GType object_type,
                       GType g_type,
                       GBoxedCopyFunc g_dup_func,
                       GDestroyNotify g_destroy_func,
                       guint hold_msec,
                       DequeuedCallback callback,
                       gpointer callback_target,
                       GeeEqualDataFunc equal_func,
                       gpointer equal_func_target,
                       GDestroyNotify equal_func_target_destroy_notify,
                       gint priority)
{
	TimedQueue* self = NULL;
	SortedList* _tmp4_;
#line 65 "../src/TimedQueue.vala"
	self = (TimedQueue*) g_type_create_instance (object_type);
#line 65 "../src/TimedQueue.vala"
	self->priv->g_type = g_type;
#line 65 "../src/TimedQueue.vala"
	self->priv->g_dup_func = g_dup_func;
#line 65 "../src/TimedQueue.vala"
	self->priv->g_destroy_func = g_destroy_func;
#line 67 "../src/TimedQueue.vala"
	self->priv->hold_msec = hold_msec;
#line 68 "../src/TimedQueue.vala"
	self->priv->callback = callback;
#line 68 "../src/TimedQueue.vala"
	self->priv->callback_target = callback_target;
#line 70 "../src/TimedQueue.vala"
	if (equal_func != NULL) {
#line 258 "TimedQueue.c"
		GeeEqualDataFunc _tmp0_;
		gpointer _tmp0__target;
		GDestroyNotify _tmp0__target_destroy_notify;
#line 71 "../src/TimedQueue.vala"
		_tmp0_ = equal_func;
#line 71 "../src/TimedQueue.vala"
		_tmp0__target = equal_func_target;
#line 71 "../src/TimedQueue.vala"
		_tmp0__target_destroy_notify = equal_func_target_destroy_notify;
#line 71 "../src/TimedQueue.vala"
		equal_func = NULL;
#line 71 "../src/TimedQueue.vala"
		equal_func_target = NULL;
#line 71 "../src/TimedQueue.vala"
		equal_func_target_destroy_notify = NULL;
#line 71 "../src/TimedQueue.vala"
		(self->priv->equal_func_target_destroy_notify == NULL) ? NULL : (self->priv->equal_func_target_destroy_notify (self->priv->equal_func_target), NULL);
#line 71 "../src/TimedQueue.vala"
		self->priv->equal_func = NULL;
#line 71 "../src/TimedQueue.vala"
		self->priv->equal_func_target = NULL;
#line 71 "../src/TimedQueue.vala"
		self->priv->equal_func_target_destroy_notify = NULL;
#line 71 "../src/TimedQueue.vala"
		self->priv->equal_func = _tmp0_;
#line 71 "../src/TimedQueue.vala"
		self->priv->equal_func_target = _tmp0__target;
#line 71 "../src/TimedQueue.vala"
		self->priv->equal_func_target_destroy_notify = _tmp0__target_destroy_notify;
#line 288 "TimedQueue.c"
	} else {
		gpointer _tmp1_ = NULL;
		GDestroyNotify _tmp2_ = NULL;
		GeeEqualDataFunc _tmp3_;
#line 73 "../src/TimedQueue.vala"
		_tmp3_ = gee_functions_get_equal_func_for (g_type, &_tmp1_, &_tmp2_);
#line 73 "../src/TimedQueue.vala"
		(self->priv->equal_func_target_destroy_notify == NULL) ? NULL : (self->priv->equal_func_target_destroy_notify (self->priv->equal_func_target), NULL);
#line 73 "../src/TimedQueue.vala"
		self->priv->equal_func = NULL;
#line 73 "../src/TimedQueue.vala"
		self->priv->equal_func_target = NULL;
#line 73 "../src/TimedQueue.vala"
		self->priv->equal_func_target_destroy_notify = NULL;
#line 73 "../src/TimedQueue.vala"
		self->priv->equal_func = (GeeEqualDataFunc) _tmp3_;
#line 73 "../src/TimedQueue.vala"
		self->priv->equal_func_target = _tmp1_;
#line 73 "../src/TimedQueue.vala"
		self->priv->equal_func_target_destroy_notify = _tmp2_;
#line 309 "TimedQueue.c"
	}
#line 75 "../src/TimedQueue.vala"
	self->priv->priority = priority;
#line 77 "../src/TimedQueue.vala"
	_tmp4_ = sorted_list_new (TIMED_QUEUE_TYPE_ELEMENT, (GBoxedCopyFunc) timed_queue_element_ref, (GDestroyNotify) timed_queue_element_unref, _timed_queue_element_comparator_comparator, NULL);
#line 77 "../src/TimedQueue.vala"
	_g_object_unref0 (self->priv->queue);
#line 77 "../src/TimedQueue.vala"
	self->priv->queue = _tmp4_;
#line 79 "../src/TimedQueue.vala"
	self->priv->timer_id = g_timeout_add_full (priority, timed_queue_get_heartbeat_timeout (self), _timed_queue_on_heartbeat_gsource_func, timed_queue_ref (self), timed_queue_unref);
#line 65 "../src/TimedQueue.vala"
	(equal_func_target_destroy_notify == NULL) ? NULL : (equal_func_target_destroy_notify (equal_func_target), NULL);
#line 65 "../src/TimedQueue.vala"
	equal_func = NULL;
#line 65 "../src/TimedQueue.vala"
	equal_func_target = NULL;
#line 65 "../src/TimedQueue.vala"
	equal_func_target_destroy_notify = NULL;
#line 65 "../src/TimedQueue.vala"
	return self;
#line 331 "TimedQueue.c"
}

TimedQueue*
timed_queue_new (GType g_type,
                 GBoxedCopyFunc g_dup_func,
                 GDestroyNotify g_destroy_func,
                 guint hold_msec,
                 DequeuedCallback callback,
                 gpointer callback_target,
                 GeeEqualDataFunc equal_func,
                 gpointer equal_func_target,
                 GDestroyNotify equal_func_target_destroy_notify,
                 gint priority)
{
#line 65 "../src/TimedQueue.vala"
	return timed_queue_construct (TYPE_TIMED_QUEUE, g_type, g_dup_func, g_destroy_func, hold_msec, callback, callback_target, equal_func, equal_func_target, equal_func_target_destroy_notify, priority);
#line 348 "TimedQueue.c"
}

guint
timed_queue_get_dequeue_spacing_msec (TimedQueue* self)
{
	guint result;
#line 87 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 88 "../src/TimedQueue.vala"
	result = self->priv->dequeue_spacing_msec;
#line 88 "../src/TimedQueue.vala"
	return result;
#line 361 "TimedQueue.c"
}

void
timed_queue_set_dequeue_spacing_msec (TimedQueue* self,
                                      guint msec)
{
#line 91 "../src/TimedQueue.vala"
	g_return_if_fail (self != NULL);
#line 92 "../src/TimedQueue.vala"
	if (msec == self->priv->dequeue_spacing_msec) {
#line 93 "../src/TimedQueue.vala"
		return;
#line 374 "TimedQueue.c"
	}
#line 95 "../src/TimedQueue.vala"
	if (self->priv->timer_id != ((guint) 0)) {
#line 96 "../src/TimedQueue.vala"
		g_source_remove (self->priv->timer_id);
#line 380 "TimedQueue.c"
	}
#line 98 "../src/TimedQueue.vala"
	self->priv->dequeue_spacing_msec = msec;
#line 100 "../src/TimedQueue.vala"
	self->priv->timer_id = g_timeout_add_full (self->priv->priority, timed_queue_get_heartbeat_timeout (self), _timed_queue_on_heartbeat_gsource_func, timed_queue_ref (self), timed_queue_unref);
#line 386 "TimedQueue.c"
}

static guint
timed_queue_get_heartbeat_timeout (TimedQueue* self)
{
	guint _tmp0_ = 0U;
	guint result;
#line 103 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, 0U);
#line 104 "../src/TimedQueue.vala"
	if (self->priv->dequeue_spacing_msec == ((guint) 0)) {
#line 105 "../src/TimedQueue.vala"
		_tmp0_ = self->priv->hold_msec / 10;
#line 400 "TimedQueue.c"
	} else {
#line 106 "../src/TimedQueue.vala"
		_tmp0_ = self->priv->dequeue_spacing_msec / 2;
#line 404 "TimedQueue.c"
	}
#line 104 "../src/TimedQueue.vala"
	result = CLAMP (_tmp0_, (guint) 10, G_MAXUINT);
#line 104 "../src/TimedQueue.vala"
	return result;
#line 410 "TimedQueue.c"
}

static void
timed_queue_real_notify_dequeued (TimedQueue* self,
                                  gconstpointer item)
{
	DequeuedCallback _tmp0_;
	gpointer _tmp0__target;
#line 110 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->callback;
#line 110 "../src/TimedQueue.vala"
	_tmp0__target = self->priv->callback_target;
#line 110 "../src/TimedQueue.vala"
	_tmp0_ (item, _tmp0__target);
#line 425 "TimedQueue.c"
}

void
timed_queue_notify_dequeued (TimedQueue* self,
                             gconstpointer item)
{
	TimedQueueClass* _klass_;
#line 109 "../src/TimedQueue.vala"
	g_return_if_fail (self != NULL);
#line 109 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 109 "../src/TimedQueue.vala"
	if (_klass_->notify_dequeued) {
#line 109 "../src/TimedQueue.vala"
		_klass_->notify_dequeued (self, item);
#line 441 "TimedQueue.c"
	}
}

gboolean
timed_queue_is_paused (TimedQueue* self)
{
	gboolean result;
#line 113 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 114 "../src/TimedQueue.vala"
	result = self->priv->paused_state;
#line 114 "../src/TimedQueue.vala"
	return result;
#line 455 "TimedQueue.c"
}

void
timed_queue_pause (TimedQueue* self)
{
#line 117 "../src/TimedQueue.vala"
	g_return_if_fail (self != NULL);
#line 118 "../src/TimedQueue.vala"
	if (self->priv->paused_state) {
#line 119 "../src/TimedQueue.vala"
		return;
#line 467 "TimedQueue.c"
	}
#line 121 "../src/TimedQueue.vala"
	self->priv->paused_state = TRUE;
#line 123 "../src/TimedQueue.vala"
	g_signal_emit (self, timed_queue_signals[TIMED_QUEUE_PAUSED_SIGNAL], 0, TRUE);
#line 473 "TimedQueue.c"
}

void
timed_queue_unpause (TimedQueue* self)
{
#line 126 "../src/TimedQueue.vala"
	g_return_if_fail (self != NULL);
#line 127 "../src/TimedQueue.vala"
	if (!self->priv->paused_state) {
#line 128 "../src/TimedQueue.vala"
		return;
#line 485 "TimedQueue.c"
	}
#line 130 "../src/TimedQueue.vala"
	self->priv->paused_state = FALSE;
#line 132 "../src/TimedQueue.vala"
	g_signal_emit (self, timed_queue_signals[TIMED_QUEUE_PAUSED_SIGNAL], 0, FALSE);
#line 491 "TimedQueue.c"
}

static void
timed_queue_real_clear (TimedQueue* self)
{
	SortedList* _tmp0_;
#line 136 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->queue;
#line 136 "../src/TimedQueue.vala"
	gee_collection_clear ((GeeCollection*) _tmp0_);
#line 502 "TimedQueue.c"
}

void
timed_queue_clear (TimedQueue* self)
{
	TimedQueueClass* _klass_;
#line 135 "../src/TimedQueue.vala"
	g_return_if_fail (self != NULL);
#line 135 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 135 "../src/TimedQueue.vala"
	if (_klass_->clear) {
#line 135 "../src/TimedQueue.vala"
		_klass_->clear (self);
#line 517 "TimedQueue.c"
	}
}

static gboolean
timed_queue_real_contains (TimedQueue* self,
                           gconstpointer item)
{
	gboolean result;
	{
		GeeIterator* _e_it = NULL;
		SortedList* _tmp0_;
		GeeIterator* _tmp1_;
#line 140 "../src/TimedQueue.vala"
		_tmp0_ = self->priv->queue;
#line 140 "../src/TimedQueue.vala"
		_tmp1_ = gee_iterable_iterator ((GeeIterable*) _tmp0_);
#line 140 "../src/TimedQueue.vala"
		_e_it = _tmp1_;
#line 140 "../src/TimedQueue.vala"
		while (TRUE) {
#line 538 "TimedQueue.c"
			GeeIterator* _tmp2_;
			TimedQueueElement* e = NULL;
			GeeIterator* _tmp3_;
			gpointer _tmp4_;
			GeeEqualDataFunc _tmp5_;
			gpointer _tmp5__target;
			TimedQueueElement* _tmp6_;
			gconstpointer _tmp7_;
#line 140 "../src/TimedQueue.vala"
			_tmp2_ = _e_it;
#line 140 "../src/TimedQueue.vala"
			if (!gee_iterator_next (_tmp2_)) {
#line 140 "../src/TimedQueue.vala"
				break;
#line 553 "TimedQueue.c"
			}
#line 140 "../src/TimedQueue.vala"
			_tmp3_ = _e_it;
#line 140 "../src/TimedQueue.vala"
			_tmp4_ = gee_iterator_get (_tmp3_);
#line 140 "../src/TimedQueue.vala"
			e = (TimedQueueElement*) _tmp4_;
#line 141 "../src/TimedQueue.vala"
			_tmp5_ = self->priv->equal_func;
#line 141 "../src/TimedQueue.vala"
			_tmp5__target = self->priv->equal_func_target;
#line 141 "../src/TimedQueue.vala"
			_tmp6_ = e;
#line 141 "../src/TimedQueue.vala"
			_tmp7_ = _tmp6_->item;
#line 141 "../src/TimedQueue.vala"
			if (_tmp5_ (item, _tmp7_, _tmp5__target)) {
#line 142 "../src/TimedQueue.vala"
				result = TRUE;
#line 142 "../src/TimedQueue.vala"
				_timed_queue_element_unref0 (e);
#line 142 "../src/TimedQueue.vala"
				_g_object_unref0 (_e_it);
#line 142 "../src/TimedQueue.vala"
				return result;
#line 579 "TimedQueue.c"
			}
#line 140 "../src/TimedQueue.vala"
			_timed_queue_element_unref0 (e);
#line 583 "TimedQueue.c"
		}
#line 140 "../src/TimedQueue.vala"
		_g_object_unref0 (_e_it);
#line 587 "TimedQueue.c"
	}
#line 145 "../src/TimedQueue.vala"
	result = FALSE;
#line 145 "../src/TimedQueue.vala"
	return result;
#line 593 "TimedQueue.c"
}

gboolean
timed_queue_contains (TimedQueue* self,
                      gconstpointer item)
{
	TimedQueueClass* _klass_;
#line 139 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 139 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 139 "../src/TimedQueue.vala"
	if (_klass_->contains) {
#line 139 "../src/TimedQueue.vala"
		return _klass_->contains (self, item);
#line 609 "TimedQueue.c"
	}
#line 139 "../src/TimedQueue.vala"
	return FALSE;
#line 613 "TimedQueue.c"
}

static gboolean
timed_queue_real_enqueue (TimedQueue* self,
                          gconstpointer item)
{
	SortedList* _tmp0_;
	TimedQueueElement* _tmp1_;
	TimedQueueElement* _tmp2_;
	gboolean _tmp3_;
	gboolean result;
#line 149 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->queue;
#line 149 "../src/TimedQueue.vala"
	_tmp1_ = timed_queue_element_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, item, timed_queue_calc_ready_time (self));
#line 149 "../src/TimedQueue.vala"
	_tmp2_ = _tmp1_;
#line 149 "../src/TimedQueue.vala"
	_tmp3_ = gee_collection_add ((GeeCollection*) _tmp0_, _tmp2_);
#line 149 "../src/TimedQueue.vala"
	_timed_queue_element_unref0 (_tmp2_);
#line 149 "../src/TimedQueue.vala"
	result = _tmp3_;
#line 149 "../src/TimedQueue.vala"
	return result;
#line 639 "TimedQueue.c"
}

gboolean
timed_queue_enqueue (TimedQueue* self,
                     gconstpointer item)
{
	TimedQueueClass* _klass_;
#line 148 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 148 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 148 "../src/TimedQueue.vala"
	if (_klass_->enqueue) {
#line 148 "../src/TimedQueue.vala"
		return _klass_->enqueue (self, item);
#line 655 "TimedQueue.c"
	}
#line 148 "../src/TimedQueue.vala"
	return FALSE;
#line 659 "TimedQueue.c"
}

static gboolean
timed_queue_real_enqueue_many (TimedQueue* self,
                               GeeCollection* items)
{
	gint64 ready_time = 0LL;
	GeeArrayList* elements = NULL;
	GeeArrayList* _tmp0_;
	SortedList* _tmp9_;
	GeeArrayList* _tmp10_;
	gboolean result;
#line 152 "../src/TimedQueue.vala"
	g_return_val_if_fail (items != NULL, FALSE);
#line 153 "../src/TimedQueue.vala"
	ready_time = timed_queue_calc_ready_time (self);
#line 155 "../src/TimedQueue.vala"
	_tmp0_ = gee_array_list_new (TIMED_QUEUE_TYPE_ELEMENT, (GBoxedCopyFunc) timed_queue_element_ref, (GDestroyNotify) timed_queue_element_unref, NULL, NULL, NULL);
#line 155 "../src/TimedQueue.vala"
	elements = _tmp0_;
#line 680 "TimedQueue.c"
	{
		GeeIterator* _item_it = NULL;
		GeeIterator* _tmp1_;
#line 156 "../src/TimedQueue.vala"
		_tmp1_ = gee_iterable_iterator ((GeeIterable*) items);
#line 156 "../src/TimedQueue.vala"
		_item_it = _tmp1_;
#line 156 "../src/TimedQueue.vala"
		while (TRUE) {
#line 690 "TimedQueue.c"
			GeeIterator* _tmp2_;
			gpointer item = NULL;
			GeeIterator* _tmp3_;
			gpointer _tmp4_;
			GeeArrayList* _tmp5_;
			gconstpointer _tmp6_;
			TimedQueueElement* _tmp7_;
			TimedQueueElement* _tmp8_;
#line 156 "../src/TimedQueue.vala"
			_tmp2_ = _item_it;
#line 156 "../src/TimedQueue.vala"
			if (!gee_iterator_next (_tmp2_)) {
#line 156 "../src/TimedQueue.vala"
				break;
#line 705 "TimedQueue.c"
			}
#line 156 "../src/TimedQueue.vala"
			_tmp3_ = _item_it;
#line 156 "../src/TimedQueue.vala"
			_tmp4_ = gee_iterator_get (_tmp3_);
#line 156 "../src/TimedQueue.vala"
			item = _tmp4_;
#line 157 "../src/TimedQueue.vala"
			_tmp5_ = elements;
#line 157 "../src/TimedQueue.vala"
			_tmp6_ = item;
#line 157 "../src/TimedQueue.vala"
			_tmp7_ = timed_queue_element_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, (GDestroyNotify) self->priv->g_destroy_func, _tmp6_, ready_time);
#line 157 "../src/TimedQueue.vala"
			_tmp8_ = _tmp7_;
#line 157 "../src/TimedQueue.vala"
			gee_abstract_collection_add ((GeeAbstractCollection*) _tmp5_, _tmp8_);
#line 157 "../src/TimedQueue.vala"
			_timed_queue_element_unref0 (_tmp8_);
#line 156 "../src/TimedQueue.vala"
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
#line 727 "TimedQueue.c"
		}
#line 156 "../src/TimedQueue.vala"
		_g_object_unref0 (_item_it);
#line 731 "TimedQueue.c"
	}
#line 159 "../src/TimedQueue.vala"
	_tmp9_ = self->priv->queue;
#line 159 "../src/TimedQueue.vala"
	_tmp10_ = elements;
#line 159 "../src/TimedQueue.vala"
	result = sorted_list_add_list (_tmp9_, (GeeList*) _tmp10_);
#line 159 "../src/TimedQueue.vala"
	_g_object_unref0 (elements);
#line 159 "../src/TimedQueue.vala"
	return result;
#line 743 "TimedQueue.c"
}

gboolean
timed_queue_enqueue_many (TimedQueue* self,
                          GeeCollection* items)
{
	TimedQueueClass* _klass_;
#line 152 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 152 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 152 "../src/TimedQueue.vala"
	if (_klass_->enqueue_many) {
#line 152 "../src/TimedQueue.vala"
		return _klass_->enqueue_many (self, items);
#line 759 "TimedQueue.c"
	}
#line 152 "../src/TimedQueue.vala"
	return FALSE;
#line 763 "TimedQueue.c"
}

static gboolean
timed_queue_real_remove_first (TimedQueue* self,
                               gconstpointer item)
{
	GeeIterator* iter = NULL;
	SortedList* _tmp0_;
	GeeIterator* _tmp1_;
	gboolean result;
#line 163 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->queue;
#line 163 "../src/TimedQueue.vala"
	_tmp1_ = gee_iterable_iterator ((GeeIterable*) _tmp0_);
#line 163 "../src/TimedQueue.vala"
	iter = _tmp1_;
#line 164 "../src/TimedQueue.vala"
	while (TRUE) {
#line 782 "TimedQueue.c"
		GeeIterator* _tmp2_;
		TimedQueueElement* e = NULL;
		GeeIterator* _tmp3_;
		gpointer _tmp4_;
		GeeEqualDataFunc _tmp5_;
		gpointer _tmp5__target;
		TimedQueueElement* _tmp6_;
		gconstpointer _tmp7_;
#line 164 "../src/TimedQueue.vala"
		_tmp2_ = iter;
#line 164 "../src/TimedQueue.vala"
		if (!gee_iterator_next (_tmp2_)) {
#line 164 "../src/TimedQueue.vala"
			break;
#line 797 "TimedQueue.c"
		}
#line 165 "../src/TimedQueue.vala"
		_tmp3_ = iter;
#line 165 "../src/TimedQueue.vala"
		_tmp4_ = gee_iterator_get (_tmp3_);
#line 165 "../src/TimedQueue.vala"
		e = (TimedQueueElement*) _tmp4_;
#line 166 "../src/TimedQueue.vala"
		_tmp5_ = self->priv->equal_func;
#line 166 "../src/TimedQueue.vala"
		_tmp5__target = self->priv->equal_func_target;
#line 166 "../src/TimedQueue.vala"
		_tmp6_ = e;
#line 166 "../src/TimedQueue.vala"
		_tmp7_ = _tmp6_->item;
#line 166 "../src/TimedQueue.vala"
		if (_tmp5_ (item, _tmp7_, _tmp5__target)) {
#line 815 "TimedQueue.c"
			GeeIterator* _tmp8_;
#line 167 "../src/TimedQueue.vala"
			_tmp8_ = iter;
#line 167 "../src/TimedQueue.vala"
			gee_iterator_remove (_tmp8_);
#line 169 "../src/TimedQueue.vala"
			result = TRUE;
#line 169 "../src/TimedQueue.vala"
			_timed_queue_element_unref0 (e);
#line 169 "../src/TimedQueue.vala"
			_g_object_unref0 (iter);
#line 169 "../src/TimedQueue.vala"
			return result;
#line 829 "TimedQueue.c"
		}
#line 164 "../src/TimedQueue.vala"
		_timed_queue_element_unref0 (e);
#line 833 "TimedQueue.c"
	}
#line 173 "../src/TimedQueue.vala"
	result = FALSE;
#line 173 "../src/TimedQueue.vala"
	_g_object_unref0 (iter);
#line 173 "../src/TimedQueue.vala"
	return result;
#line 841 "TimedQueue.c"
}

gboolean
timed_queue_remove_first (TimedQueue* self,
                          gconstpointer item)
{
	TimedQueueClass* _klass_;
#line 162 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 162 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 162 "../src/TimedQueue.vala"
	if (_klass_->remove_first) {
#line 162 "../src/TimedQueue.vala"
		return _klass_->remove_first (self, item);
#line 857 "TimedQueue.c"
	}
#line 162 "../src/TimedQueue.vala"
	return FALSE;
#line 861 "TimedQueue.c"
}

static gint64
timed_queue_calc_ready_time (TimedQueue* self)
{
	gint64 result;
#line 182 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, 0LL);
#line 183 "../src/TimedQueue.vala"
	result = now_ms () + self->priv->hold_msec;
#line 183 "../src/TimedQueue.vala"
	return result;
#line 874 "TimedQueue.c"
}

static gboolean
timed_queue_on_heartbeat (TimedQueue* self)
{
	gint64 now = 0LL;
	gboolean result;
#line 186 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 187 "../src/TimedQueue.vala"
	if (self->priv->paused_state) {
#line 188 "../src/TimedQueue.vala"
		result = TRUE;
#line 188 "../src/TimedQueue.vala"
		return result;
#line 890 "TimedQueue.c"
	}
#line 190 "../src/TimedQueue.vala"
	now = (gint64) 0;
#line 894 "TimedQueue.c"
	{
		gboolean _tmp0_ = FALSE;
#line 192 "../src/TimedQueue.vala"
		_tmp0_ = TRUE;
#line 192 "../src/TimedQueue.vala"
		while (TRUE) {
#line 901 "TimedQueue.c"
			SortedList* _tmp1_;
			gint _tmp2_;
			gint _tmp3_;
			TimedQueueElement* head = NULL;
			SortedList* _tmp4_;
			gpointer _tmp5_;
			TimedQueueElement* _tmp6_;
			TimedQueueElement* _tmp7_;
			gboolean _tmp8_ = FALSE;
			TimedQueueElement* h = NULL;
			SortedList* _tmp9_;
			gpointer _tmp10_;
			TimedQueueElement* _tmp11_;
			TimedQueueElement* _tmp12_;
			TimedQueueElement* _tmp13_;
			gconstpointer _tmp14_;
#line 192 "../src/TimedQueue.vala"
			if (!_tmp0_) {
#line 920 "TimedQueue.c"
			}
#line 192 "../src/TimedQueue.vala"
			_tmp0_ = FALSE;
#line 193 "../src/TimedQueue.vala"
			_tmp1_ = self->priv->queue;
#line 193 "../src/TimedQueue.vala"
			_tmp2_ = gee_collection_get_size ((GeeCollection*) _tmp1_);
#line 193 "../src/TimedQueue.vala"
			_tmp3_ = _tmp2_;
#line 193 "../src/TimedQueue.vala"
			if (_tmp3_ == 0) {
#line 194 "../src/TimedQueue.vala"
				break;
#line 934 "TimedQueue.c"
			}
#line 196 "../src/TimedQueue.vala"
			_tmp4_ = self->priv->queue;
#line 196 "../src/TimedQueue.vala"
			_tmp5_ = sorted_list_get_at (_tmp4_, 0);
#line 196 "../src/TimedQueue.vala"
			head = (TimedQueueElement*) _tmp5_;
#line 197 "../src/TimedQueue.vala"
			_tmp6_ = head;
#line 197 "../src/TimedQueue.vala"
			_vala_assert (_tmp6_ != NULL, "head != null");
#line 199 "../src/TimedQueue.vala"
			if (now == ((gint64) 0)) {
#line 200 "../src/TimedQueue.vala"
				now = now_ms ();
#line 950 "TimedQueue.c"
			}
#line 202 "../src/TimedQueue.vala"
			_tmp7_ = head;
#line 202 "../src/TimedQueue.vala"
			if (_tmp7_->ready > now) {
#line 203 "../src/TimedQueue.vala"
				_timed_queue_element_unref0 (head);
#line 203 "../src/TimedQueue.vala"
				break;
#line 960 "TimedQueue.c"
			}
#line 206 "../src/TimedQueue.vala"
			if (self->priv->dequeue_spacing_msec != ((guint) 0)) {
#line 206 "../src/TimedQueue.vala"
				_tmp8_ = (now - self->priv->last_dequeue) < ((gint64) self->priv->dequeue_spacing_msec);
#line 966 "TimedQueue.c"
			} else {
#line 206 "../src/TimedQueue.vala"
				_tmp8_ = FALSE;
#line 970 "TimedQueue.c"
			}
#line 206 "../src/TimedQueue.vala"
			if (_tmp8_) {
#line 207 "../src/TimedQueue.vala"
				_timed_queue_element_unref0 (head);
#line 207 "../src/TimedQueue.vala"
				break;
#line 978 "TimedQueue.c"
			}
#line 209 "../src/TimedQueue.vala"
			_tmp9_ = self->priv->queue;
#line 209 "../src/TimedQueue.vala"
			_tmp10_ = sorted_list_remove_at (_tmp9_, 0);
#line 209 "../src/TimedQueue.vala"
			h = (TimedQueueElement*) _tmp10_;
#line 210 "../src/TimedQueue.vala"
			_tmp11_ = head;
#line 210 "../src/TimedQueue.vala"
			_tmp12_ = h;
#line 210 "../src/TimedQueue.vala"
			_vala_assert (_tmp11_ == _tmp12_, "head == h");
#line 212 "../src/TimedQueue.vala"
			_tmp13_ = head;
#line 212 "../src/TimedQueue.vala"
			_tmp14_ = _tmp13_->item;
#line 212 "../src/TimedQueue.vala"
			timed_queue_notify_dequeued (self, _tmp14_);
#line 213 "../src/TimedQueue.vala"
			self->priv->last_dequeue = now;
#line 217 "../src/TimedQueue.vala"
			if (self->priv->dequeue_spacing_msec != ((guint) 0)) {
#line 218 "../src/TimedQueue.vala"
				_timed_queue_element_unref0 (h);
#line 218 "../src/TimedQueue.vala"
				_timed_queue_element_unref0 (head);
#line 218 "../src/TimedQueue.vala"
				break;
#line 1008 "TimedQueue.c"
			}
#line 192 "../src/TimedQueue.vala"
			_timed_queue_element_unref0 (h);
#line 192 "../src/TimedQueue.vala"
			_timed_queue_element_unref0 (head);
#line 1014 "TimedQueue.c"
		}
	}
#line 221 "../src/TimedQueue.vala"
	result = TRUE;
#line 221 "../src/TimedQueue.vala"
	return result;
#line 1021 "TimedQueue.c"
}

gint
timed_queue_get_size (TimedQueue* self)
{
	TimedQueueClass* _klass_;
#line 177 "../src/TimedQueue.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 177 "../src/TimedQueue.vala"
	_klass_ = TIMED_QUEUE_GET_CLASS (self);
#line 177 "../src/TimedQueue.vala"
	if (_klass_->get_size) {
#line 177 "../src/TimedQueue.vala"
		return _klass_->get_size (self);
#line 1036 "TimedQueue.c"
	}
#line 177 "../src/TimedQueue.vala"
	return -1;
#line 1040 "TimedQueue.c"
}

static gint
timed_queue_real_get_size (TimedQueue* base)
{
	gint result;
	TimedQueue* self;
	SortedList* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
#line 177 "../src/TimedQueue.vala"
	self = base;
#line 178 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->queue;
#line 178 "../src/TimedQueue.vala"
	_tmp1_ = gee_collection_get_size ((GeeCollection*) _tmp0_);
#line 178 "../src/TimedQueue.vala"
	_tmp2_ = _tmp1_;
#line 178 "../src/TimedQueue.vala"
	result = _tmp2_;
#line 178 "../src/TimedQueue.vala"
	return result;
#line 1063 "TimedQueue.c"
}

static void
timed_queue_real_paused (TimedQueue* self,
                         gboolean is_paused)
{
}

static inline gpointer
timed_queue_element_get_instance_private (TimedQueueElement* self)
{
	return G_STRUCT_MEMBER_P (self, TimedQueueElement_private_offset);
}

static TimedQueueElement*
timed_queue_element_construct (GType object_type,
                               GType g_type,
                               GBoxedCopyFunc g_dup_func,
                               GDestroyNotify g_destroy_func,
                               gconstpointer item,
                               gint64 ready)
{
	TimedQueueElement* self = NULL;
	gpointer _tmp0_;
#line 39 "../src/TimedQueue.vala"
	self = (TimedQueueElement*) g_type_create_instance (object_type);
#line 39 "../src/TimedQueue.vala"
	self->priv->g_type = g_type;
#line 39 "../src/TimedQueue.vala"
	self->priv->g_dup_func = g_dup_func;
#line 39 "../src/TimedQueue.vala"
	self->priv->g_destroy_func = g_destroy_func;
#line 40 "../src/TimedQueue.vala"
	_tmp0_ = ((item != NULL) && (g_dup_func != NULL)) ? g_dup_func ((gpointer) item) : ((gpointer) item);
#line 40 "../src/TimedQueue.vala"
	((self->item == NULL) || (g_destroy_func == NULL)) ? NULL : (self->item = (g_destroy_func (self->item), NULL));
#line 40 "../src/TimedQueue.vala"
	self->item = _tmp0_;
#line 41 "../src/TimedQueue.vala"
	self->ready = ready;
#line 39 "../src/TimedQueue.vala"
	return self;
#line 1106 "TimedQueue.c"
}

static TimedQueueElement*
timed_queue_element_new (GType g_type,
                         GBoxedCopyFunc g_dup_func,
                         GDestroyNotify g_destroy_func,
                         gconstpointer item,
                         gint64 ready)
{
#line 39 "../src/TimedQueue.vala"
	return timed_queue_element_construct (TIMED_QUEUE_TYPE_ELEMENT, g_type, g_dup_func, g_destroy_func, item, ready);
#line 1118 "TimedQueue.c"
}

static gint64
timed_queue_element_comparator (void* a,
                                void* b)
{
	gint64 result;
#line 45 "../src/TimedQueue.vala"
	result = ((TimedQueueElement*) a)->ready - ((TimedQueueElement*) b)->ready;
#line 45 "../src/TimedQueue.vala"
	return result;
#line 1130 "TimedQueue.c"
}

static void
timed_queue_value_element_init (GValue* value)
{
#line 35 "../src/TimedQueue.vala"
	value->data[0].v_pointer = NULL;
#line 1138 "TimedQueue.c"
}

static void
timed_queue_value_element_free_value (GValue* value)
{
#line 35 "../src/TimedQueue.vala"
	if (value->data[0].v_pointer) {
#line 35 "../src/TimedQueue.vala"
		timed_queue_element_unref (value->data[0].v_pointer);
#line 1148 "TimedQueue.c"
	}
}

static void
timed_queue_value_element_copy_value (const GValue* src_value,
                                      GValue* dest_value)
{
#line 35 "../src/TimedQueue.vala"
	if (src_value->data[0].v_pointer) {
#line 35 "../src/TimedQueue.vala"
		dest_value->data[0].v_pointer = timed_queue_element_ref (src_value->data[0].v_pointer);
#line 1160 "TimedQueue.c"
	} else {
#line 35 "../src/TimedQueue.vala"
		dest_value->data[0].v_pointer = NULL;
#line 1164 "TimedQueue.c"
	}
}

static gpointer
timed_queue_value_element_peek_pointer (const GValue* value)
{
#line 35 "../src/TimedQueue.vala"
	return value->data[0].v_pointer;
#line 1173 "TimedQueue.c"
}

static gchar*
timed_queue_value_element_collect_value (GValue* value,
                                         guint n_collect_values,
                                         GTypeCValue* collect_values,
                                         guint collect_flags)
{
#line 35 "../src/TimedQueue.vala"
	if (collect_values[0].v_pointer) {
#line 1184 "TimedQueue.c"
		TimedQueueElement * object;
		object = collect_values[0].v_pointer;
#line 35 "../src/TimedQueue.vala"
		if (object->parent_instance.g_class == NULL) {
#line 35 "../src/TimedQueue.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 1191 "TimedQueue.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 35 "../src/TimedQueue.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 1195 "TimedQueue.c"
		}
#line 35 "../src/TimedQueue.vala"
		value->data[0].v_pointer = timed_queue_element_ref (object);
#line 1199 "TimedQueue.c"
	} else {
#line 35 "../src/TimedQueue.vala"
		value->data[0].v_pointer = NULL;
#line 1203 "TimedQueue.c"
	}
#line 35 "../src/TimedQueue.vala"
	return NULL;
#line 1207 "TimedQueue.c"
}

static gchar*
timed_queue_value_element_lcopy_value (const GValue* value,
                                       guint n_collect_values,
                                       GTypeCValue* collect_values,
                                       guint collect_flags)
{
	TimedQueueElement ** object_p;
	object_p = collect_values[0].v_pointer;
#line 35 "../src/TimedQueue.vala"
	if (!object_p) {
#line 35 "../src/TimedQueue.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 1222 "TimedQueue.c"
	}
#line 35 "../src/TimedQueue.vala"
	if (!value->data[0].v_pointer) {
#line 35 "../src/TimedQueue.vala"
		*object_p = NULL;
#line 1228 "TimedQueue.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 35 "../src/TimedQueue.vala"
		*object_p = value->data[0].v_pointer;
#line 1232 "TimedQueue.c"
	} else {
#line 35 "../src/TimedQueue.vala"
		*object_p = timed_queue_element_ref (value->data[0].v_pointer);
#line 1236 "TimedQueue.c"
	}
#line 35 "../src/TimedQueue.vala"
	return NULL;
#line 1240 "TimedQueue.c"
}

static GParamSpec*
timed_queue_param_spec_element (const gchar* name,
                                const gchar* nick,
                                const gchar* blurb,
                                GType object_type,
                                GParamFlags flags)
{
	TimedQueueParamSpecElement* spec;
#line 35 "../src/TimedQueue.vala"
	g_return_val_if_fail (g_type_is_a (object_type, TIMED_QUEUE_TYPE_ELEMENT), NULL);
#line 35 "../src/TimedQueue.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 35 "../src/TimedQueue.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 35 "../src/TimedQueue.vala"
	return G_PARAM_SPEC (spec);
#line 1259 "TimedQueue.c"
}

static gpointer
timed_queue_value_get_element (const GValue* value)
{
#line 35 "../src/TimedQueue.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT), NULL);
#line 35 "../src/TimedQueue.vala"
	return value->data[0].v_pointer;
#line 1269 "TimedQueue.c"
}

static void
timed_queue_value_set_element (GValue* value,
                               gpointer v_object)
{
	TimedQueueElement * old;
#line 35 "../src/TimedQueue.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT));
#line 35 "../src/TimedQueue.vala"
	old = value->data[0].v_pointer;
#line 35 "../src/TimedQueue.vala"
	if (v_object) {
#line 35 "../src/TimedQueue.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TIMED_QUEUE_TYPE_ELEMENT));
#line 35 "../src/TimedQueue.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 35 "../src/TimedQueue.vala"
		value->data[0].v_pointer = v_object;
#line 35 "../src/TimedQueue.vala"
		timed_queue_element_ref (value->data[0].v_pointer);
#line 1291 "TimedQueue.c"
	} else {
#line 35 "../src/TimedQueue.vala"
		value->data[0].v_pointer = NULL;
#line 1295 "TimedQueue.c"
	}
#line 35 "../src/TimedQueue.vala"
	if (old) {
#line 35 "../src/TimedQueue.vala"
		timed_queue_element_unref (old);
#line 1301 "TimedQueue.c"
	}
}

static void
timed_queue_value_take_element (GValue* value,
                                gpointer v_object)
{
	TimedQueueElement * old;
#line 35 "../src/TimedQueue.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TIMED_QUEUE_TYPE_ELEMENT));
#line 35 "../src/TimedQueue.vala"
	old = value->data[0].v_pointer;
#line 35 "../src/TimedQueue.vala"
	if (v_object) {
#line 35 "../src/TimedQueue.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TIMED_QUEUE_TYPE_ELEMENT));
#line 35 "../src/TimedQueue.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 35 "../src/TimedQueue.vala"
		value->data[0].v_pointer = v_object;
#line 1322 "TimedQueue.c"
	} else {
#line 35 "../src/TimedQueue.vala"
		value->data[0].v_pointer = NULL;
#line 1326 "TimedQueue.c"
	}
#line 35 "../src/TimedQueue.vala"
	if (old) {
#line 35 "../src/TimedQueue.vala"
		timed_queue_element_unref (old);
#line 1332 "TimedQueue.c"
	}
}

static void
timed_queue_element_class_init (TimedQueueElementClass * klass,
                                gpointer klass_data)
{
#line 35 "../src/TimedQueue.vala"
	timed_queue_element_parent_class = g_type_class_peek_parent (klass);
#line 35 "../src/TimedQueue.vala"
	((TimedQueueElementClass *) klass)->finalize = timed_queue_element_finalize;
#line 35 "../src/TimedQueue.vala"
	g_type_class_adjust_private_offset (klass, &TimedQueueElement_private_offset);
#line 1346 "TimedQueue.c"
}

static void
timed_queue_element_instance_init (TimedQueueElement * self,
                                   gpointer klass)
{
#line 35 "../src/TimedQueue.vala"
	self->priv = timed_queue_element_get_instance_private (self);
#line 35 "../src/TimedQueue.vala"
	self->ref_count = 1;
#line 1357 "TimedQueue.c"
}

static void
timed_queue_element_finalize (TimedQueueElement * obj)
{
	TimedQueueElement * self;
#line 35 "../src/TimedQueue.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TIMED_QUEUE_TYPE_ELEMENT, TimedQueueElement);
#line 35 "../src/TimedQueue.vala"
	g_signal_handlers_destroy (self);
#line 36 "../src/TimedQueue.vala"
	((self->item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (self->item = (self->priv->g_destroy_func (self->item), NULL));
#line 1370 "TimedQueue.c"
}

static GType
timed_queue_element_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { timed_queue_value_element_init, timed_queue_value_element_free_value, timed_queue_value_element_copy_value, timed_queue_value_element_peek_pointer, "p", timed_queue_value_element_collect_value, "p", timed_queue_value_element_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (TimedQueueElementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) timed_queue_element_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TimedQueueElement), 0, (GInstanceInitFunc) timed_queue_element_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType timed_queue_element_type_id;
	timed_queue_element_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TimedQueueElement", &g_define_type_info, &g_define_type_fundamental_info, 0);
	TimedQueueElement_private_offset = g_type_add_instance_private (timed_queue_element_type_id, sizeof (TimedQueueElementPrivate));
	return timed_queue_element_type_id;
}

static GType
timed_queue_element_get_type (void)
{
	static volatile gsize timed_queue_element_type_id__once = 0;
	if (g_once_init_enter (&timed_queue_element_type_id__once)) {
		GType timed_queue_element_type_id;
		timed_queue_element_type_id = timed_queue_element_get_type_once ();
		g_once_init_leave (&timed_queue_element_type_id__once, timed_queue_element_type_id);
	}
	return timed_queue_element_type_id__once;
}

static gpointer
timed_queue_element_ref (gpointer instance)
{
	TimedQueueElement * self;
	self = instance;
#line 35 "../src/TimedQueue.vala"
	g_atomic_int_inc (&self->ref_count);
#line 35 "../src/TimedQueue.vala"
	return instance;
#line 1406 "TimedQueue.c"
}

static void
timed_queue_element_unref (gpointer instance)
{
	TimedQueueElement * self;
	self = instance;
#line 35 "../src/TimedQueue.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 35 "../src/TimedQueue.vala"
		TIMED_QUEUE_ELEMENT_GET_CLASS (self)->finalize (self);
#line 35 "../src/TimedQueue.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 1420 "TimedQueue.c"
	}
}

static void
value_timed_queue_init (GValue* value)
{
#line 34 "../src/TimedQueue.vala"
	value->data[0].v_pointer = NULL;
#line 1429 "TimedQueue.c"
}

static void
value_timed_queue_free_value (GValue* value)
{
#line 34 "../src/TimedQueue.vala"
	if (value->data[0].v_pointer) {
#line 34 "../src/TimedQueue.vala"
		timed_queue_unref (value->data[0].v_pointer);
#line 1439 "TimedQueue.c"
	}
}

static void
value_timed_queue_copy_value (const GValue* src_value,
                              GValue* dest_value)
{
#line 34 "../src/TimedQueue.vala"
	if (src_value->data[0].v_pointer) {
#line 34 "../src/TimedQueue.vala"
		dest_value->data[0].v_pointer = timed_queue_ref (src_value->data[0].v_pointer);
#line 1451 "TimedQueue.c"
	} else {
#line 34 "../src/TimedQueue.vala"
		dest_value->data[0].v_pointer = NULL;
#line 1455 "TimedQueue.c"
	}
}

static gpointer
value_timed_queue_peek_pointer (const GValue* value)
{
#line 34 "../src/TimedQueue.vala"
	return value->data[0].v_pointer;
#line 1464 "TimedQueue.c"
}

static gchar*
value_timed_queue_collect_value (GValue* value,
                                 guint n_collect_values,
                                 GTypeCValue* collect_values,
                                 guint collect_flags)
{
#line 34 "../src/TimedQueue.vala"
	if (collect_values[0].v_pointer) {
#line 1475 "TimedQueue.c"
		TimedQueue * object;
		object = collect_values[0].v_pointer;
#line 34 "../src/TimedQueue.vala"
		if (object->parent_instance.g_class == NULL) {
#line 34 "../src/TimedQueue.vala"
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 1482 "TimedQueue.c"
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
#line 34 "../src/TimedQueue.vala"
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
#line 1486 "TimedQueue.c"
		}
#line 34 "../src/TimedQueue.vala"
		value->data[0].v_pointer = timed_queue_ref (object);
#line 1490 "TimedQueue.c"
	} else {
#line 34 "../src/TimedQueue.vala"
		value->data[0].v_pointer = NULL;
#line 1494 "TimedQueue.c"
	}
#line 34 "../src/TimedQueue.vala"
	return NULL;
#line 1498 "TimedQueue.c"
}

static gchar*
value_timed_queue_lcopy_value (const GValue* value,
                               guint n_collect_values,
                               GTypeCValue* collect_values,
                               guint collect_flags)
{
	TimedQueue ** object_p;
	object_p = collect_values[0].v_pointer;
#line 34 "../src/TimedQueue.vala"
	if (!object_p) {
#line 34 "../src/TimedQueue.vala"
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
#line 1513 "TimedQueue.c"
	}
#line 34 "../src/TimedQueue.vala"
	if (!value->data[0].v_pointer) {
#line 34 "../src/TimedQueue.vala"
		*object_p = NULL;
#line 1519 "TimedQueue.c"
	} else if (collect_flags & G_VALUE_NOCOPY_CONTENTS) {
#line 34 "../src/TimedQueue.vala"
		*object_p = value->data[0].v_pointer;
#line 1523 "TimedQueue.c"
	} else {
#line 34 "../src/TimedQueue.vala"
		*object_p = timed_queue_ref (value->data[0].v_pointer);
#line 1527 "TimedQueue.c"
	}
#line 34 "../src/TimedQueue.vala"
	return NULL;
#line 1531 "TimedQueue.c"
}

GParamSpec*
param_spec_timed_queue (const gchar* name,
                        const gchar* nick,
                        const gchar* blurb,
                        GType object_type,
                        GParamFlags flags)
{
	ParamSpecTimedQueue* spec;
#line 34 "../src/TimedQueue.vala"
	g_return_val_if_fail (g_type_is_a (object_type, TYPE_TIMED_QUEUE), NULL);
#line 34 "../src/TimedQueue.vala"
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
#line 34 "../src/TimedQueue.vala"
	G_PARAM_SPEC (spec)->value_type = object_type;
#line 34 "../src/TimedQueue.vala"
	return G_PARAM_SPEC (spec);
#line 1550 "TimedQueue.c"
}

gpointer
value_get_timed_queue (const GValue* value)
{
#line 34 "../src/TimedQueue.vala"
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE), NULL);
#line 34 "../src/TimedQueue.vala"
	return value->data[0].v_pointer;
#line 1560 "TimedQueue.c"
}

void
value_set_timed_queue (GValue* value,
                       gpointer v_object)
{
	TimedQueue * old;
#line 34 "../src/TimedQueue.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE));
#line 34 "../src/TimedQueue.vala"
	old = value->data[0].v_pointer;
#line 34 "../src/TimedQueue.vala"
	if (v_object) {
#line 34 "../src/TimedQueue.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TIMED_QUEUE));
#line 34 "../src/TimedQueue.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 34 "../src/TimedQueue.vala"
		value->data[0].v_pointer = v_object;
#line 34 "../src/TimedQueue.vala"
		timed_queue_ref (value->data[0].v_pointer);
#line 1582 "TimedQueue.c"
	} else {
#line 34 "../src/TimedQueue.vala"
		value->data[0].v_pointer = NULL;
#line 1586 "TimedQueue.c"
	}
#line 34 "../src/TimedQueue.vala"
	if (old) {
#line 34 "../src/TimedQueue.vala"
		timed_queue_unref (old);
#line 1592 "TimedQueue.c"
	}
}

void
value_take_timed_queue (GValue* value,
                        gpointer v_object)
{
	TimedQueue * old;
#line 34 "../src/TimedQueue.vala"
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, TYPE_TIMED_QUEUE));
#line 34 "../src/TimedQueue.vala"
	old = value->data[0].v_pointer;
#line 34 "../src/TimedQueue.vala"
	if (v_object) {
#line 34 "../src/TimedQueue.vala"
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, TYPE_TIMED_QUEUE));
#line 34 "../src/TimedQueue.vala"
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
#line 34 "../src/TimedQueue.vala"
		value->data[0].v_pointer = v_object;
#line 1613 "TimedQueue.c"
	} else {
#line 34 "../src/TimedQueue.vala"
		value->data[0].v_pointer = NULL;
#line 1617 "TimedQueue.c"
	}
#line 34 "../src/TimedQueue.vala"
	if (old) {
#line 34 "../src/TimedQueue.vala"
		timed_queue_unref (old);
#line 1623 "TimedQueue.c"
	}
}

static void
timed_queue_class_init (TimedQueueClass * klass,
                        gpointer klass_data)
{
#line 34 "../src/TimedQueue.vala"
	timed_queue_parent_class = g_type_class_peek_parent (klass);
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->finalize = timed_queue_finalize;
#line 34 "../src/TimedQueue.vala"
	g_type_class_adjust_private_offset (klass, &TimedQueue_private_offset);
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->notify_dequeued = (void (*) (TimedQueue*, gconstpointer)) timed_queue_real_notify_dequeued;
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->clear = (void (*) (TimedQueue*)) timed_queue_real_clear;
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->contains = (gboolean (*) (TimedQueue*, gconstpointer)) timed_queue_real_contains;
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->enqueue = (gboolean (*) (TimedQueue*, gconstpointer)) timed_queue_real_enqueue;
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->enqueue_many = (gboolean (*) (TimedQueue*, GeeCollection*)) timed_queue_real_enqueue_many;
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->remove_first = (gboolean (*) (TimedQueue*, gconstpointer)) timed_queue_real_remove_first;
#line 34 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->paused = timed_queue_real_paused;
#line 34 "../src/TimedQueue.vala"
	TIMED_QUEUE_CLASS (klass)->get_size = (gint (*) (TimedQueue*)) timed_queue_real_get_size;
#line 34 "../src/TimedQueue.vala"
	timed_queue_signals[TIMED_QUEUE_PAUSED_SIGNAL] = g_signal_new ("paused", TYPE_TIMED_QUEUE, G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (TimedQueueClass, paused), NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
#line 1655 "TimedQueue.c"
}

static void
timed_queue_instance_init (TimedQueue * self,
                           gpointer klass)
{
#line 34 "../src/TimedQueue.vala"
	self->priv = timed_queue_get_instance_private (self);
#line 53 "../src/TimedQueue.vala"
	self->priv->timer_id = (guint) 0;
#line 55 "../src/TimedQueue.vala"
	self->priv->dequeue_spacing_msec = (guint) 0;
#line 56 "../src/TimedQueue.vala"
	self->priv->last_dequeue = (gint64) 0;
#line 57 "../src/TimedQueue.vala"
	self->priv->paused_state = FALSE;
#line 34 "../src/TimedQueue.vala"
	self->ref_count = 1;
#line 1674 "TimedQueue.c"
}

static void
timed_queue_finalize (TimedQueue * obj)
{
	TimedQueue * self;
#line 34 "../src/TimedQueue.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_TIMED_QUEUE, TimedQueue);
#line 34 "../src/TimedQueue.vala"
	g_signal_handlers_destroy (self);
#line 83 "../src/TimedQueue.vala"
	if (self->priv->timer_id != ((guint) 0)) {
#line 84 "../src/TimedQueue.vala"
		g_source_remove (self->priv->timer_id);
#line 1689 "TimedQueue.c"
	}
#line 51 "../src/TimedQueue.vala"
	(self->priv->equal_func_target_destroy_notify == NULL) ? NULL : (self->priv->equal_func_target_destroy_notify (self->priv->equal_func_target), NULL);
#line 51 "../src/TimedQueue.vala"
	self->priv->equal_func = NULL;
#line 51 "../src/TimedQueue.vala"
	self->priv->equal_func_target = NULL;
#line 51 "../src/TimedQueue.vala"
	self->priv->equal_func_target_destroy_notify = NULL;
#line 54 "../src/TimedQueue.vala"
	_g_object_unref0 (self->priv->queue);
#line 1701 "TimedQueue.c"
}

static GType
timed_queue_get_type_once (void)
{
	static const GTypeValueTable g_define_type_value_table = { value_timed_queue_init, value_timed_queue_free_value, value_timed_queue_copy_value, value_timed_queue_peek_pointer, "p", value_timed_queue_collect_value, "p", value_timed_queue_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (TimedQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) timed_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (TimedQueue), 0, (GInstanceInitFunc) timed_queue_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	GType timed_queue_type_id;
	timed_queue_type_id = g_type_register_fundamental (g_type_fundamental_next (), "TimedQueue", &g_define_type_info, &g_define_type_fundamental_info, 0);
	TimedQueue_private_offset = g_type_add_instance_private (timed_queue_type_id, sizeof (TimedQueuePrivate));
	return timed_queue_type_id;
}

GType
timed_queue_get_type (void)
{
	static volatile gsize timed_queue_type_id__once = 0;
	if (g_once_init_enter (&timed_queue_type_id__once)) {
		GType timed_queue_type_id;
		timed_queue_type_id = timed_queue_get_type_once ();
		g_once_init_leave (&timed_queue_type_id__once, timed_queue_type_id);
	}
	return timed_queue_type_id__once;
}

gpointer
timed_queue_ref (gpointer instance)
{
	TimedQueue * self;
	self = instance;
#line 34 "../src/TimedQueue.vala"
	g_atomic_int_inc (&self->ref_count);
#line 34 "../src/TimedQueue.vala"
	return instance;
#line 1737 "TimedQueue.c"
}

void
timed_queue_unref (gpointer instance)
{
	TimedQueue * self;
	self = instance;
#line 34 "../src/TimedQueue.vala"
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
#line 34 "../src/TimedQueue.vala"
		TIMED_QUEUE_GET_CLASS (self)->finalize (self);
#line 34 "../src/TimedQueue.vala"
		g_type_free_instance ((GTypeInstance *) self);
#line 1751 "TimedQueue.c"
	}
}

static inline gpointer
hash_timed_queue_get_instance_private (HashTimedQueue* self)
{
	return G_STRUCT_MEMBER_P (self, HashTimedQueue_private_offset);
}

HashTimedQueue*
hash_timed_queue_construct (GType object_type,
                            GType g_type,
                            GBoxedCopyFunc g_dup_func,
                            GDestroyNotify g_destroy_func,
                            guint hold_msec,
                            DequeuedCallback callback,
                            gpointer callback_target,
                            GeeHashDataFunc hash_func,
                            gpointer hash_func_target,
                            GDestroyNotify hash_func_target_destroy_notify,
                            GeeEqualDataFunc equal_func,
                            gpointer equal_func_target,
                            GDestroyNotify equal_func_target_destroy_notify,
                            gint priority)
{
	HashTimedQueue* self = NULL;
	GeeEqualDataFunc _tmp0_;
	gpointer _tmp0__target;
	GDestroyNotify _tmp0__target_destroy_notify;
	GeeHashDataFunc _tmp1_;
	gpointer _tmp1__target;
	GDestroyNotify _tmp1__target_destroy_notify;
	GeeEqualDataFunc _tmp2_;
	gpointer _tmp2__target;
	GDestroyNotify _tmp2__target_destroy_notify;
	GeeHashMap* _tmp3_;
#line 233 "../src/TimedQueue.vala"
	_tmp0_ = equal_func;
#line 233 "../src/TimedQueue.vala"
	_tmp0__target = equal_func_target;
#line 233 "../src/TimedQueue.vala"
	_tmp0__target_destroy_notify = equal_func_target_destroy_notify;
#line 233 "../src/TimedQueue.vala"
	equal_func = NULL;
#line 233 "../src/TimedQueue.vala"
	equal_func_target = NULL;
#line 233 "../src/TimedQueue.vala"
	equal_func_target_destroy_notify = NULL;
#line 233 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) timed_queue_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, hold_msec, callback, callback_target, _tmp0_, _tmp0__target, _tmp0__target_destroy_notify, priority);
#line 233 "../src/TimedQueue.vala"
	self->priv->g_type = g_type;
#line 233 "../src/TimedQueue.vala"
	self->priv->g_dup_func = g_dup_func;
#line 233 "../src/TimedQueue.vala"
	self->priv->g_destroy_func = g_destroy_func;
#line 235 "../src/TimedQueue.vala"
	_tmp1_ = hash_func;
#line 235 "../src/TimedQueue.vala"
	_tmp1__target = hash_func_target;
#line 235 "../src/TimedQueue.vala"
	_tmp1__target_destroy_notify = hash_func_target_destroy_notify;
#line 235 "../src/TimedQueue.vala"
	hash_func = NULL;
#line 235 "../src/TimedQueue.vala"
	hash_func_target = NULL;
#line 235 "../src/TimedQueue.vala"
	hash_func_target_destroy_notify = NULL;
#line 235 "../src/TimedQueue.vala"
	_tmp2_ = equal_func;
#line 235 "../src/TimedQueue.vala"
	_tmp2__target = equal_func_target;
#line 235 "../src/TimedQueue.vala"
	_tmp2__target_destroy_notify = equal_func_target_destroy_notify;
#line 235 "../src/TimedQueue.vala"
	equal_func = NULL;
#line 235 "../src/TimedQueue.vala"
	equal_func_target = NULL;
#line 235 "../src/TimedQueue.vala"
	equal_func_target_destroy_notify = NULL;
#line 235 "../src/TimedQueue.vala"
	_tmp3_ = gee_hash_map_new (g_type, (GBoxedCopyFunc) g_dup_func, (GDestroyNotify) g_destroy_func, G_TYPE_INT, NULL, NULL, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify, _tmp2_, _tmp2__target, _tmp2__target_destroy_notify, NULL, NULL, NULL);
#line 235 "../src/TimedQueue.vala"
	_g_object_unref0 (self->priv->item_count);
#line 235 "../src/TimedQueue.vala"
	self->priv->item_count = _tmp3_;
#line 230 "../src/TimedQueue.vala"
	(hash_func_target_destroy_notify == NULL) ? NULL : (hash_func_target_destroy_notify (hash_func_target), NULL);
#line 230 "../src/TimedQueue.vala"
	hash_func = NULL;
#line 230 "../src/TimedQueue.vala"
	hash_func_target = NULL;
#line 230 "../src/TimedQueue.vala"
	hash_func_target_destroy_notify = NULL;
#line 230 "../src/TimedQueue.vala"
	(equal_func_target_destroy_notify == NULL) ? NULL : (equal_func_target_destroy_notify (equal_func_target), NULL);
#line 230 "../src/TimedQueue.vala"
	equal_func = NULL;
#line 230 "../src/TimedQueue.vala"
	equal_func_target = NULL;
#line 230 "../src/TimedQueue.vala"
	equal_func_target_destroy_notify = NULL;
#line 230 "../src/TimedQueue.vala"
	return self;
#line 1856 "TimedQueue.c"
}

HashTimedQueue*
hash_timed_queue_new (GType g_type,
                      GBoxedCopyFunc g_dup_func,
                      GDestroyNotify g_destroy_func,
                      guint hold_msec,
                      DequeuedCallback callback,
                      gpointer callback_target,
                      GeeHashDataFunc hash_func,
                      gpointer hash_func_target,
                      GDestroyNotify hash_func_target_destroy_notify,
                      GeeEqualDataFunc equal_func,
                      gpointer equal_func_target,
                      GDestroyNotify equal_func_target_destroy_notify,
                      gint priority)
{
#line 230 "../src/TimedQueue.vala"
	return hash_timed_queue_construct (TYPE_HASH_TIMED_QUEUE, g_type, g_dup_func, g_destroy_func, hold_msec, callback, callback_target, hash_func, hash_func_target, hash_func_target_destroy_notify, equal_func, equal_func_target, equal_func_target_destroy_notify, priority);
#line 1876 "TimedQueue.c"
}

static void
hash_timed_queue_real_notify_dequeued (TimedQueue* base,
                                       gconstpointer item)
{
	HashTimedQueue * self;
#line 238 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) base;
#line 239 "../src/TimedQueue.vala"
	hash_timed_queue_removed (self, item);
#line 241 "../src/TimedQueue.vala"
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->notify_dequeued (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_TIMED_QUEUE, TimedQueue), item);
#line 1890 "TimedQueue.c"
}

static void
hash_timed_queue_real_clear (TimedQueue* base)
{
	HashTimedQueue * self;
	GeeHashMap* _tmp0_;
#line 244 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) base;
#line 245 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->item_count;
#line 245 "../src/TimedQueue.vala"
	gee_abstract_map_clear ((GeeAbstractMap*) _tmp0_);
#line 247 "../src/TimedQueue.vala"
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->clear (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_TIMED_QUEUE, TimedQueue));
#line 1906 "TimedQueue.c"
}

static gboolean
hash_timed_queue_real_contains (TimedQueue* base,
                                gconstpointer item)
{
	HashTimedQueue * self;
	GeeHashMap* _tmp0_;
	gboolean result;
#line 250 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) base;
#line 251 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->item_count;
#line 251 "../src/TimedQueue.vala"
	result = gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, item);
#line 251 "../src/TimedQueue.vala"
	return result;
#line 1924 "TimedQueue.c"
}

static gboolean
hash_timed_queue_real_enqueue (TimedQueue* base,
                               gconstpointer item)
{
	HashTimedQueue * self;
	gint _tmp0_ = 0;
	GeeHashMap* _tmp1_;
	GeeHashMap* _tmp4_;
	gboolean result;
#line 254 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) base;
#line 255 "../src/TimedQueue.vala"
	if (!TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->enqueue (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_TIMED_QUEUE, TimedQueue), item)) {
#line 256 "../src/TimedQueue.vala"
		result = FALSE;
#line 256 "../src/TimedQueue.vala"
		return result;
#line 1944 "TimedQueue.c"
	}
#line 258 "../src/TimedQueue.vala"
	_tmp1_ = self->priv->item_count;
#line 258 "../src/TimedQueue.vala"
	if (gee_abstract_map_has_key ((GeeAbstractMap*) _tmp1_, item)) {
#line 1950 "TimedQueue.c"
		GeeHashMap* _tmp2_;
		gpointer _tmp3_;
#line 258 "../src/TimedQueue.vala"
		_tmp2_ = self->priv->item_count;
#line 258 "../src/TimedQueue.vala"
		_tmp3_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp2_, item);
#line 258 "../src/TimedQueue.vala"
		_tmp0_ = ((gint) ((gintptr) _tmp3_)) + 1;
#line 1959 "TimedQueue.c"
	} else {
#line 258 "../src/TimedQueue.vala"
		_tmp0_ = 1;
#line 1963 "TimedQueue.c"
	}
#line 258 "../src/TimedQueue.vala"
	_tmp4_ = self->priv->item_count;
#line 258 "../src/TimedQueue.vala"
	gee_abstract_map_set ((GeeAbstractMap*) _tmp4_, item, (gpointer) ((gintptr) _tmp0_));
#line 260 "../src/TimedQueue.vala"
	result = TRUE;
#line 260 "../src/TimedQueue.vala"
	return result;
#line 1973 "TimedQueue.c"
}

static gboolean
hash_timed_queue_real_enqueue_many (TimedQueue* base,
                                    GeeCollection* items)
{
	HashTimedQueue * self;
	gboolean result;
#line 263 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) base;
#line 263 "../src/TimedQueue.vala"
	g_return_val_if_fail (items != NULL, FALSE);
#line 264 "../src/TimedQueue.vala"
	if (!TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->enqueue_many (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_TIMED_QUEUE, TimedQueue), items)) {
#line 265 "../src/TimedQueue.vala"
		result = FALSE;
#line 265 "../src/TimedQueue.vala"
		return result;
#line 1992 "TimedQueue.c"
	}
	{
		GeeIterator* _item_it = NULL;
		GeeIterator* _tmp0_;
#line 267 "../src/TimedQueue.vala"
		_tmp0_ = gee_iterable_iterator ((GeeIterable*) items);
#line 267 "../src/TimedQueue.vala"
		_item_it = _tmp0_;
#line 267 "../src/TimedQueue.vala"
		while (TRUE) {
#line 2003 "TimedQueue.c"
			GeeIterator* _tmp1_;
			gpointer item = NULL;
			GeeIterator* _tmp2_;
			gpointer _tmp3_;
			gint _tmp4_ = 0;
			GeeHashMap* _tmp5_;
			gconstpointer _tmp6_;
			GeeHashMap* _tmp10_;
			gconstpointer _tmp11_;
#line 267 "../src/TimedQueue.vala"
			_tmp1_ = _item_it;
#line 267 "../src/TimedQueue.vala"
			if (!gee_iterator_next (_tmp1_)) {
#line 267 "../src/TimedQueue.vala"
				break;
#line 2019 "TimedQueue.c"
			}
#line 267 "../src/TimedQueue.vala"
			_tmp2_ = _item_it;
#line 267 "../src/TimedQueue.vala"
			_tmp3_ = gee_iterator_get (_tmp2_);
#line 267 "../src/TimedQueue.vala"
			item = _tmp3_;
#line 268 "../src/TimedQueue.vala"
			_tmp5_ = self->priv->item_count;
#line 268 "../src/TimedQueue.vala"
			_tmp6_ = item;
#line 268 "../src/TimedQueue.vala"
			if (gee_abstract_map_has_key ((GeeAbstractMap*) _tmp5_, _tmp6_)) {
#line 2033 "TimedQueue.c"
				GeeHashMap* _tmp7_;
				gconstpointer _tmp8_;
				gpointer _tmp9_;
#line 268 "../src/TimedQueue.vala"
				_tmp7_ = self->priv->item_count;
#line 268 "../src/TimedQueue.vala"
				_tmp8_ = item;
#line 268 "../src/TimedQueue.vala"
				_tmp9_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp7_, _tmp8_);
#line 268 "../src/TimedQueue.vala"
				_tmp4_ = ((gint) ((gintptr) _tmp9_)) + 1;
#line 2045 "TimedQueue.c"
			} else {
#line 268 "../src/TimedQueue.vala"
				_tmp4_ = 1;
#line 2049 "TimedQueue.c"
			}
#line 268 "../src/TimedQueue.vala"
			_tmp10_ = self->priv->item_count;
#line 268 "../src/TimedQueue.vala"
			_tmp11_ = item;
#line 268 "../src/TimedQueue.vala"
			gee_abstract_map_set ((GeeAbstractMap*) _tmp10_, _tmp11_, (gpointer) ((gintptr) _tmp4_));
#line 267 "../src/TimedQueue.vala"
			((item == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (item = (self->priv->g_destroy_func (item), NULL));
#line 2059 "TimedQueue.c"
		}
#line 267 "../src/TimedQueue.vala"
		_g_object_unref0 (_item_it);
#line 2063 "TimedQueue.c"
	}
#line 270 "../src/TimedQueue.vala"
	result = TRUE;
#line 270 "../src/TimedQueue.vala"
	return result;
#line 2069 "TimedQueue.c"
}

static gboolean
hash_timed_queue_real_remove_first (TimedQueue* base,
                                    gconstpointer item)
{
	HashTimedQueue * self;
	gboolean result;
#line 273 "../src/TimedQueue.vala"
	self = (HashTimedQueue*) base;
#line 274 "../src/TimedQueue.vala"
	if (!TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->remove_first (G_TYPE_CHECK_INSTANCE_CAST (self, TYPE_TIMED_QUEUE, TimedQueue), item)) {
#line 275 "../src/TimedQueue.vala"
		result = FALSE;
#line 275 "../src/TimedQueue.vala"
		return result;
#line 2086 "TimedQueue.c"
	}
#line 277 "../src/TimedQueue.vala"
	hash_timed_queue_removed (self, item);
#line 279 "../src/TimedQueue.vala"
	result = TRUE;
#line 279 "../src/TimedQueue.vala"
	return result;
#line 2094 "TimedQueue.c"
}

static void
hash_timed_queue_removed (HashTimedQueue* self,
                          gconstpointer item)
{
	GeeHashMap* _tmp0_;
	gint count = 0;
	GeeHashMap* _tmp1_;
	gpointer _tmp2_;
	gint _tmp3_;
#line 282 "../src/TimedQueue.vala"
	g_return_if_fail (self != NULL);
#line 285 "../src/TimedQueue.vala"
	_tmp0_ = self->priv->item_count;
#line 285 "../src/TimedQueue.vala"
	if (!gee_abstract_map_has_key ((GeeAbstractMap*) _tmp0_, item)) {
#line 286 "../src/TimedQueue.vala"
		return;
#line 2114 "TimedQueue.c"
	}
#line 288 "../src/TimedQueue.vala"
	_tmp1_ = self->priv->item_count;
#line 288 "../src/TimedQueue.vala"
	_tmp2_ = gee_abstract_map_get ((GeeAbstractMap*) _tmp1_, item);
#line 288 "../src/TimedQueue.vala"
	count = (gint) ((gintptr) _tmp2_);
#line 289 "../src/TimedQueue.vala"
	_vala_assert (count > 0, "count > 0");
#line 291 "../src/TimedQueue.vala"
	count = count - 1;
#line 291 "../src/TimedQueue.vala"
	_tmp3_ = count;
#line 291 "../src/TimedQueue.vala"
	if (_tmp3_ == 0) {
#line 2130 "TimedQueue.c"
		GeeHashMap* _tmp4_;
#line 292 "../src/TimedQueue.vala"
		_tmp4_ = self->priv->item_count;
#line 292 "../src/TimedQueue.vala"
		gee_abstract_map_unset ((GeeAbstractMap*) _tmp4_, item, NULL);
#line 2136 "TimedQueue.c"
	} else {
		GeeHashMap* _tmp5_;
#line 294 "../src/TimedQueue.vala"
		_tmp5_ = self->priv->item_count;
#line 294 "../src/TimedQueue.vala"
		gee_abstract_map_set ((GeeAbstractMap*) _tmp5_, item, (gpointer) ((gintptr) count));
#line 2143 "TimedQueue.c"
	}
}

static void
hash_timed_queue_class_init (HashTimedQueueClass * klass,
                             gpointer klass_data)
{
#line 227 "../src/TimedQueue.vala"
	hash_timed_queue_parent_class = g_type_class_peek_parent (klass);
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->finalize = hash_timed_queue_finalize;
#line 227 "../src/TimedQueue.vala"
	g_type_class_adjust_private_offset (klass, &HashTimedQueue_private_offset);
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->notify_dequeued = (void (*) (TimedQueue*, gconstpointer)) hash_timed_queue_real_notify_dequeued;
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->clear = (void (*) (TimedQueue*)) hash_timed_queue_real_clear;
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->contains = (gboolean (*) (TimedQueue*, gconstpointer)) hash_timed_queue_real_contains;
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->enqueue = (gboolean (*) (TimedQueue*, gconstpointer)) hash_timed_queue_real_enqueue;
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->enqueue_many = (gboolean (*) (TimedQueue*, GeeCollection*)) hash_timed_queue_real_enqueue_many;
#line 227 "../src/TimedQueue.vala"
	((TimedQueueClass *) klass)->remove_first = (gboolean (*) (TimedQueue*, gconstpointer)) hash_timed_queue_real_remove_first;
#line 2169 "TimedQueue.c"
}

static void
hash_timed_queue_instance_init (HashTimedQueue * self,
                                gpointer klass)
{
#line 227 "../src/TimedQueue.vala"
	self->priv = hash_timed_queue_get_instance_private (self);
#line 2178 "TimedQueue.c"
}

static void
hash_timed_queue_finalize (TimedQueue * obj)
{
	HashTimedQueue * self;
#line 227 "../src/TimedQueue.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_HASH_TIMED_QUEUE, HashTimedQueue);
#line 228 "../src/TimedQueue.vala"
	_g_object_unref0 (self->priv->item_count);
#line 227 "../src/TimedQueue.vala"
	TIMED_QUEUE_CLASS (hash_timed_queue_parent_class)->finalize (obj);
#line 2191 "TimedQueue.c"
}

static GType
hash_timed_queue_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (HashTimedQueueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) hash_timed_queue_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (HashTimedQueue), 0, (GInstanceInitFunc) hash_timed_queue_instance_init, NULL };
	GType hash_timed_queue_type_id;
	hash_timed_queue_type_id = g_type_register_static (TYPE_TIMED_QUEUE, "HashTimedQueue", &g_define_type_info, 0);
	HashTimedQueue_private_offset = g_type_add_instance_private (hash_timed_queue_type_id, sizeof (HashTimedQueuePrivate));
	return hash_timed_queue_type_id;
}

GType
hash_timed_queue_get_type (void)
{
	static volatile gsize hash_timed_queue_type_id__once = 0;
	if (g_once_init_enter (&hash_timed_queue_type_id__once)) {
		GType hash_timed_queue_type_id;
		hash_timed_queue_type_id = hash_timed_queue_get_type_once ();
		g_once_init_leave (&hash_timed_queue_type_id__once, hash_timed_queue_type_id);
	}
	return hash_timed_queue_type_id__once;
}

